<?php
require_once DOC_ROOT.DS.'protected'.DS.'config'.DS.'base.config.php';
require_once gURL('config').DS.'namespace.config.php';
require_once gURL('config').DS.'route.config.php';
require_once gURL('config').DS.'db.config.php';
require_once gURL('config').DS.'lang.config.php';

/**
 * EVENING STAR is my nick name on network. Short name is named as evening.
 * Bootstrap class
 * @package base
 * @since 1.0
 * @author JJB <splitedragon@hotmail.com>
 * @copyright Copyright (c) 2019, EVENING
 */
class Evening
{
    /**
     * Class autoload loader.
     * Support both namespace and non-namespace
     *
     * This method is provided to be invoked within an __autoload() magic method.
     * e.g. HomeController class is placed at /controllers/homecontroller.php
     * !important class name is case-insensitive on Windows, HomeController, HoMeController both is find  at /controllers/homecontroller.php
     * @param string $class class name
     * @return boolean whether the class has been loaded successfully
     */
    public static function autoload($class)
    {
        if(strpos($class, '\\') === false) { // non-namespace
            return self::classMapLoad($class);
        }
        else {
            return self::namespaceMapLoad($class);
        }

        return false;
    }

    private static function classMapLoad($class)
    {
        $map = gClassMap();
        foreach ($map as $value) {
            $root = $value;
            //$path = $root.DS.strtolower($class).'.php';
            $path = $root.DS.$class.'.php'; // For case-sensitive of UNIX
            if (file_exists($path))
            {
                require_once $path;
                return true;
            }
        }

        if(DEBUG == 1) die ("Cannot found: $class");
        return false;
    }

    private static function namespaceMapLoad($class)
    {
        $fileName = '';
        $namespace = '';
        $className = '';
        // Sets the include default path as the "vendors" directory
        $includePath = NAMESPACE_BASEPATH;

        if (false !== ($lastNsPos = strripos($class, '\\'))) {
            $namespace = substr($class, 0, $lastNsPos);
            $className = substr($class, $lastNsPos + 1);

            $map = gNamespaceMap();
            foreach ($map as $key=>$val) {
                if(strpos($namespace, $key) === 0) {
                    $namespace = str_replace($key, $val, $namespace);
                    $includePath = '';
                }
            }

            $fileName = str_replace('\\', DS, $namespace).DS;
        }
        $fileName .= $className.'.php'; //str_replace('_', DS, $className).'.php';

        $path = $includePath!=''? gURL('protected').DS.$includePath.DS.$fileName : gURL('protected').DS.$fileName;

        if (file_exists($path))
        {
            require_once $path;
            return true;
        }

        if(DEBUG == 1) die ("Cannot found: $class: $path");
        return false;
    }

    /**
     * Boot loader
     */
    public static function run()
    {
        $_urls = Router::prepare();

        Middleware::run();

        $path       = $_urls['path'];
        $controller = $_urls['controller'];
        $action     = $_urls['action'];
        $argument   = $_urls['argument'];

        $instance = Loader::loadController($path.DS.$controller);
        try{
            $instance->_runAction($action, $argument);
        }
        catch (Exception $e) {
            $page = new Errors('500');
            $page->message = $e->getMessage();
            $page->show();
        }
        catch (Throwable $t) {
            $page = new Errors('500');
            $page->message = $t->getMessage();
            $page->show();
        }
    }
}

spl_autoload_register(array('Evening','autoload'));