<?php
/**
 * Loader base class
 * @package base
 * @since 1.0
 * @author JJB <splitedragon@hotmail.com>
 * @copyright Copyright (c) 2019, EVENING
 */
class Loader
{
    public static function loadFile($filePath)
    {
        if(!file_exists($filePath)) {
            $tmp = explode('protected', $filePath);
            return 'File not exist: '.isset($tmp[1])? $tmp[1] : FileUtils::getBasename($filePath);
        }

        require_once $filePath;

        return true;
    }

    public static function loadClass($filePath, $className='')
    {
        $result = self::loadFile($filePath);
        if($result !== TRUE) {
            return $result;
        }
        try{
            if($className=='') {
                $className = ucwords(pathinfo($filePath, PATHINFO_FILENAME));
            }
            $instance = new $className();
            return $instance;
        }
        catch (Exception $e) {
            return $e->getMessage();
        }
        catch (Throwable $t) {
            return $t->getMessage();
        }
    }

    public static function loadController($path)
    {
        $r = Router::controller($path);
        $filePath = $r['filePath'];
        $className = $r['className'];

        $instance = self::loadClass($filePath, $className);
        if(!is_object($instance) || !($instance instanceof Controller)) {
            if(DEBUG) {
                $err = new Errors(500);
                $err->message = "Controller exception: " . $instance;
                $err->show();
            }
            exit;
        } else {
            return $instance;
        }
    }

    public static function loadModel($path)
    {
        $r = Router::model($path);
        $filePath = $r['filePath'];
        $className = $r['className'];

        $instance = self::loadClass($filePath, $className);
        if(!is_object($instance) || !($instance instanceof Model)) {
            if(DEBUG) {
                $err = new Errors(500);
                $err->message = "Model exception: " . $instance;
                $err->show();
            }
            exit;
        } else {
            return $instance;
        }
    }

    public static function loadView($path)
    {
        $r = Router::view($path);
        $filePath = $r['filePath'];
        $className = $r['className'];

        if(!file_exists($filePath)) {
            $tmp = explode('protected', $filePath);
            $err = new Errors(500);
            $err->message = 'File not exist: '.isset($tmp[1])? $tmp[1] : FileUtils::getBasename($filePath);
            $err->show();
        }

        $instance = new View($filePath);
        if(!is_object($instance) || !($instance instanceof View)) {
            if(DEBUG) {
                $err = new Errors(500);
                $err->message = "View exception: " . $instance;
                $err->show();
            }
            exit;
        } else {
            return $instance;
        }
    }
}